@@ -1,7 +1,7 @@
 /*
    Project: batmon
 
-   Copyright (C) 2006-2010 GNUstep Application Project
+   Copyright (C) 2006-2011 GNUstep Application Project
 
    Author: Riccardo Mottola 
 
@@ -38,12 +38,28 @@
 #  include <stdio.h>
 #endif
 
+#if defined(netbsd) || defined(__NetBSD__)
+#include <paths.h>  /* path for the system devices */
+#include <fcntl.h>  /* open */
+#include <unistd.h>
+#include <sys/envsys.h>
+#include <prop/proplib.h> /* psd property dictionaries */
+#endif
+
+#if defined(openbsd) || defined(__OpenBSD__)
+#include <unistd.h>
+#include <fcntl.h>  /* open */
+#include <sys/ioctl.h>
+#include <machine/apmvar.h>
+#define APMDEV "/dev/apm"
+#endif
 
 #import <Foundation/NSCharacterSet.h>
 #import <Foundation/NSString.h>
 #import <Foundation/NSArray.h>
 #import <Foundation/NSEnumerator.h>
 #import <Foundation/NSFileManager.h>
+#import <Foundation/NSException.h>
 
 #import "BatteryModel.h"
 
@@ -267,7 +283,211 @@
     chargePercent = currCap/lastCap*100;
     isCharging = NO;
   }
+#elif defined(netbsd) || defined (__NetBSD__)
+  int		sysmonfd; /* fd of /dev/sysmon */
+  int rval;
+  prop_dictionary_t bsd_dict;
+  NSString *string;
+  NSDictionary *dict;
+  char *cStr;
+  NSEnumerator *enum1;
+  id obj1;
+  NSDictionary *acpibat0;
+  NSMutableDictionary *batDict;
+  NSString *valueStr;
+  float chargeRate, dischargeRate;
+
+  /* Open the device in ro mode */
+  if ((sysmonfd = open(_PATH_SYSMON, O_RDONLY)) == -1)
+    NSLog(@"Error opening device: %s", _PATH_SYSMON);
+
+  rval = prop_dictionary_recv_ioctl(sysmonfd,
+				    ENVSYS_GETDICTIONARY,
+				    &bsd_dict);
+  if (rval)
+    {
+      NSLog(@"Error: %s", strerror(rval));
+      return;
+    }
   
+  cStr = prop_dictionary_externalize(bsd_dict);
+  string = [NSString stringWithCString: cStr];
+  dict = [string propertyList];
+  if (dict == nil)
+    {
+      NSLog(@"Could not parse dictionary");
+      return;
+    }
+
+  acpibat0 = [dict objectForKey: @"acpibat0"];
+  NSLog(@"acpibat0: %@", acpibat0);
+  NSLog(@"Class: %@", [acpibat0 class]);
+  batDict = [NSMutableDictionary dictionaryWithCapacity: 3];
+  enum1 = [acpibat0 objectEnumerator];
+  while ((obj1 = [enum1 nextObject]))
+    {
+      NSLog(@"--->%@", obj1);
+      if ([obj1 isKindOfClass: [NSDictionary class]])
+	{
+	  NSString *descriptionKey;
+
+	  descriptionKey = [obj1 objectForKey:@"description"];
+	  //	  NSLog(@"key-----> %@", descriptionKey);
+	  if (descriptionKey)
+	    [batDict setObject: obj1 forKey: descriptionKey];
+	}
+      else
+	NSLog(@"not a dict");
+
+    }
+  NSLog(@"battery dictionary: %@", batDict);
+  (void)close(sysmonfd);
+
+  valueStr = [[batDict objectForKey: @"design voltage"] objectForKey: @"cur-value"];
+  //  NSLog(@"design voltage: %@", valueStr);
+  //  if (valueStr)
+  //    designVoltage = [valueStr floatValue] / 1000000;
+
+  valueStr = [[batDict objectForKey: @"voltage"] objectForKey: @"cur-value"];
+  if (valueStr)
+    volts = [valueStr floatValue] / 1000000;
+  NSLog(@"voltage: %@ %f", valueStr, volts);
+
+  valueStr = [[batDict objectForKey: @"design cap"] objectForKey: @"cur-value"];
+  if (valueStr)
+    {
+      if ([[[batDict objectForKey: @"design cap"] objectForKey: @"type"] isEqualToString: @"Ampere hour"])
+	{
+	  useWattHours = NO;
+	}
+      else if ([[[batDict objectForKey: @"design cap"] objectForKey: @"type"] isEqualToString: @"Watt hour"])
+	{
+	  useWattHours = YES;
+	}
+      desCap = [valueStr floatValue] / 1000000;
+    }
+  NSLog(@"design cap: %@ %f", valueStr, desCap);
+
+  valueStr = [[batDict objectForKey: @"last full cap"] objectForKey: @"cur-value"];
+  if (valueStr)
+    lastCap = [valueStr floatValue] / 1000000;
+  NSLog(@"last full cap: %@, %f", valueStr, lastCap);
+
+  valueStr = [[batDict objectForKey: @"charge rate"] objectForKey: @"cur-value"];
+  chargeRate = 0;
+  if (valueStr)
+    chargeRate = [valueStr floatValue] / 1000000;
+  NSLog(@"charge rate: %@ %f", valueStr, chargeRate);
+
+  valueStr = [[batDict objectForKey: @"discharge rate"] objectForKey: @"cur-value"];
+  dischargeRate = 0;
+  if (valueStr)
+    dischargeRate = [valueStr floatValue] / 1000000;
+  NSLog(@"discharge rate: %@ %f", valueStr, dischargeRate);
+
+  valueStr = [[batDict objectForKey: @"charge"] objectForKey: @"cur-value"];
+  if (valueStr)
+    currCap = [valueStr floatValue] / 1000000;
+  NSLog(@"charge: %@, %f", valueStr, currCap);
+
+  valueStr = [[batDict objectForKey: @"charging"] objectForKey: @"cur-value"];
+  NSLog(@"charging: %@", valueStr);
+  NSLog(@"currCap %f, lastCap %f, amps %f", currCap, lastCap,amps);
+  if ([valueStr intValue] == 0)
+    {
+      amps = dischargeRate;
+      isCharging = NO;
+      if (amps > 0)
+	timeRemaining = currCap / amps;
+      else
+	timeRemaining = -1;
+      chargePercent = currCap/lastCap*100;
+
+    }
+  else
+    {
+      amps = chargeRate;
+      isCharging = YES;
+      if (amps > 0)
+	timeRemaining = (lastCap-currCap) / amps;
+      else
+	timeRemaining = -1;
+      chargePercent = currCap/lastCap*100;
+    }
+  watts = amps * volts;
+
+  valueStr = [[batDict objectForKey: @"charge state"] objectForKey: @"battery-capacity"];
+  //NSLog(@"charge state: %@", valueStr);
+  if (valueStr)
+    {
+      [chargeState release];
+      chargeState = [[NSString stringWithString: valueStr] retain];
+    }
+
+#elif defined(openbsd) || defined(__OpenBSD__)
+  int apmfd;
+  struct apm_power_info apmPwInfo;
+  BOOL validBattery;
+
+  apmfd = open(APMDEV, O_RDONLY);
+  if (apmfd == -1)
+    return;
+
+  if( -1 == ioctl(apmfd, APM_IOC_GETPOWER, &apmPwInfo) )
+    return;
+
+  isCharging = NO;
+  validBattery = YES;
+  if (APM_BATT_HIGH == apmPwInfo.battery_state)
+    chargeState = @"High";
+  else if (APM_BATT_LOW == apmPwInfo.battery_state)
+    chargeState = @"Low";
+  else if (APM_BATT_CRITICAL == apmPwInfo.battery_state)
+    chargeState = @"Critical";
+  else if (APM_BATT_CHARGING == apmPwInfo.battery_state)
+    {
+      chargeState = @"Charging";
+      isCharging = YES;
+    }
+  else if (APM_BATTERY_ABSENT == apmPwInfo.battery_state)
+    {
+      chargeState = @"Not present";
+      validBattery = NO;
+    }
+  else
+    {
+      chargeState = @"Unknown";
+      validBattery = NO;
+    }
+
+  if (APM_AC_ON == apmPwInfo.ac_state)
+    isCharging = YES;
+
+  /* we expect time in hours */
+  if (validBattery)
+    {
+      timeRemaining = (float)apmPwInfo.minutes_left / 60;
+      chargePercent = (float)(int)apmPwInfo.battery_life;
+
+      /* sanity checks */
+      if (isCharging && timeRemaining > 100)
+	timeRemaining = 0;
+      if (chargePercent > 100)
+	chargePercent = 100;
+      else if (chargePercent < 0)
+	chargePercent = 0;
+
+      if (timeRemaining < 0)
+	timeRemaining = 0;
+    }
+  else
+    {
+      chargePercent = 0;
+      timeRemaining= 0;
+    }
+
+  close(apmfd);
+
 #elif defined(linux)
 
     FILE *stateFile;
@@ -299,13 +519,16 @@
 	NSString *valueStr;
 	NSString *keyStr;
 
-	NSLog(@"reading %@", batterySysAcpiString);
-
+	//	NSLog(@"reading %@", batterySysAcpiString);
 	ueventFileName = [batterySysAcpiString stringByAppendingPathComponent:@"uevent"];
 
         [ueventFileName getCString:batteryStatePath0];
         stateFile = fopen(batteryStatePath0, "r");
-        assert(stateFile != NULL);
+	if (stateFile == NULL)
+	  {
+	    NSLog(@"acpi /sys state file is null");
+	    return;
+	  }
 
         ueventDict = [[NSMutableDictionary alloc] initWithCapacity: 4];
 
@@ -326,7 +549,7 @@
 	  }
 
 
-        NSLog(@"%@", ueventDict);
+	//        NSLog(@"%@", ueventDict);
 
 
         amps = [[ueventDict objectForKey:@"POWER_SUPPLY_CURRENT_NOW"] floatValue] / 1000000;
@@ -334,9 +557,13 @@
 	if ([ueventDict objectForKey:@"POWER_SUPPLY_POWER_NOW"] == nil)
 	  watts = volts*amps;
 	else
-	  watts = [[ueventDict objectForKey:@"POWER_SUPPLY_POWER_NOW"] floatValue] / 1000000;
-
+	  {
+	    watts = [[ueventDict objectForKey:@"POWER_SUPPLY_POWER_NOW"] floatValue] / 1000000;
+	    if (volts > 0)
+	      amps = watts / volts;
+	  }
 
+	useWattHours = YES;
 	if ([ueventDict objectForKey:@"POWER_SUPPLY_ENERGY_FULL_DESIGN"] != nil)
 	  {
 	    desCap = [[ueventDict objectForKey:@"POWER_SUPPLY_ENERGY_FULL_DESIGN"] floatValue] / 1000000;
@@ -413,7 +640,11 @@
     else if (useACPIproc)
       {
 	stateFile = fopen(batteryStatePath0, "r");
-	assert(stateFile != NULL);
+	if (stateFile == NULL)
+	  {
+	    NSLog(@"acpi /proc state file null");
+	    return;
+	  }
 
 	[self _readLine :stateFile :line];
 	sscanf(line, "present: %s", presentStr);
@@ -434,7 +665,7 @@
 	voltageVal = atoi(voltageStr);
 
 	infoFile = fopen(batteryInfoPath0, "r");
-	assert(infoFile != NULL);
+	NSAssert(infoFile != NULL, @"ACPI - /proc: info file shall not be NULL");
 
 	[self _readLine :infoFile :line];
 	sscanf(line, "present: %s", present2Str);
@@ -516,11 +747,15 @@
 
 	percentIsInvalid = NO;
 	stateFile = fopen(apmPath, "r");
-	assert(stateFile != NULL);
+	if (stateFile == NULL)
+	  {
+	    NSLog(@"apm state file null");
+	    return;
+	  }
 
 
 	[self _readLine :stateFile :line];
-	NSLog(@"line: %s", line);
+	//	NSLog(@"line: %s", line);
 	sscanf(line, "%s %s %s %s %s %s %s %s %s", drvVersionStr, apmBiosVersionStr, apmBiosFlagsStr, acLineStatusStr, battStatusStr, battFlagsStr, percentStr, timeRemainingStr, timeUnitStr);
 
 	if (percentStr != NULL && strlen(percentStr) > 0)
@@ -537,7 +772,7 @@
 		chargePercent = 0;
 		percentIsInvalid = YES;
 	      }
-	    NSLog(@"percent %f", chargePercent);
+	    //	    NSLog(@"percent %f", chargePercent);
 	  }
 
 	if (battStatusStr != NULL && strlen(battStatusStr) > 0)
@@ -589,7 +824,6 @@
 		else
 		  chargePercent = 0;
 	      }
-	    NSLog(@"percent %f", chargePercent);
 	  }
     
 	fclose(stateFile);
